@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2000,2001 Sasha Vasko <sasha at aftercode.net>
+ * Copyright (c) 2004 Valeriy Onuchin <Valeri dot Onoutchine at cern dot ch>
+ * Copyright (c) 2000-2004 Sasha Vasko <sasha at aftercode.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -63,6 +64,7 @@
 #include "asvisual.h"
 #include "blender.h"
 #include "asimage.h"
+#include "ascmap.h"
 
 static ASVisual __as_dummy_asvisual = {0};
 static ASVisual *__as_default_asvisual = &__as_dummy_asvisual ;
@@ -117,24 +119,18 @@ int mmx_init(void)
 {
 int mmx_available = 0;
 #ifdef HAVE_MMX
-	asm volatile (
-                      /* Get CPU version information */
-                      "pushl %%eax\n\t"
-                      "pushl %%ebx\n\t"
-                      "pushl %%ecx\n\t"
-                      "pushl %%edx\n\t"
+	int ebx_save;
+ 	asm volatile (
+                      "movl %%ebx, %1\n\t"
                       "movl $1, %%eax\n\t"
                       "cpuid\n\t"
-                      "andl $0x800000, %%edx \n\t"
-					  "movl %%edx, %0\n\t"
-                      "popl %%edx\n\t"
-                      "popl %%ecx\n\t"
-                      "popl %%ebx\n\t"
-                      "popl %%eax\n\t"
-                      : "=m" (mmx_available)
+                      "andl $0x800000, %%edx\n\t"	
+                      "movl %1, %%ebx\n\t"
+                      "movl %%edx, %0\n\t"
+                      : "=m" (mmx_available), "=m" (ebx_save)
                       : /* no input */
-					  : "ebx", "ecx", "edx"
-			  );
+                      : "eax", "ecx", "edx"
+                     );
 #endif
 	return mmx_available;
 }
@@ -176,6 +172,8 @@ asimage_init (ASImage * im, Bool free_resources)
 				free( im->alt.argb32 );
 			if( im->alt.vector )
 				free( im->alt.vector );
+			if( im->name ) 
+				free( im->name );
 		}
 		memset (im, 0x00, sizeof (ASImage));
 		im->magic = MAGIC_ASIMAGE ;
@@ -205,11 +203,10 @@ alloc_asimage_channels ( ASImage *im )
 {
 	/* we want result to be 32bit aligned and padded */
 	im->red = safecalloc (1, sizeof (ASStorageID) * im->height * 4);
+	LOCAL_DEBUG_OUT( "allocated %p for channels of the image %p", im->red, im );
 	if( im->red == NULL )
 	{
 		show_error( "Insufficient memory to create image %dx%d!", im->width, im->height );
-		if( im->red )
-			free( im->red );
 		return ;
 	}
 
@@ -238,6 +235,32 @@ asimage_start (ASImage * im, unsigned int width, unsigned int height, unsigned i
 	}
 }
 
+Bool
+asimage_replace (ASImage *im, ASImage *from)
+{
+	if ( im && from && im != from )
+		if( im->magic == MAGIC_ASIMAGE && from->magic == MAGIC_ASIMAGE && from->imageman == NULL )
+		{
+			int ref_count = im->ref_count ;
+			ASImageManager *imageman = im->imageman ;
+			char *name = im->name ; 
+
+			im->name = NULL ; 
+			asimage_init (im, True);
+
+			memcpy( im, from, sizeof(ASImage) );
+			memset( from, 0x00, sizeof(ASImage) );		
+
+			im->ref_count = ref_count ; 
+			im->imageman = imageman ;
+			im->name = name ;
+
+			return True ;
+		}
+	return False;
+}
+
+
 static ASImage* 
 check_created_asimage( ASImage *im, unsigned int width, unsigned int height )
 {
@@ -392,7 +415,8 @@ asimage_destroy (ASHashableValue value, void *data)
 			else
 				im->imageman = NULL ;
 		}
-		free( (char*)value );
+		if( (char*)value != im->name ) 
+			free( (char*)value );/* name */
 		destroy_asimage( &im );
 	}
 }
@@ -452,10 +476,14 @@ store_asimage( ASImageManager* imageman, ASImage *im, const char *name )
 		if( AS_ASSERT_NOTVAL(im->magic, MAGIC_ASIMAGE) )
 			im = NULL ;
 	if( !AS_ASSERT(imageman) && !AS_ASSERT(im) && !AS_ASSERT((char*)name) )
+	{
 		if( im->imageman == NULL )
 		{
 			int hash_res ;
-			im->name = mystrdup( name );
+			char *stored_name = mystrdup( name );
+			if( im->name ) 
+				free( im->name );
+			im->name = stored_name ;
 			hash_res = add_hash_item( imageman->image_hash, AS_HASHABLE(im->name), im);
 			res = ( hash_res == ASH_Success);
 			if( !res )
@@ -468,6 +496,7 @@ store_asimage( ASImageManager* imageman, ASImage *im, const char *name )
 				im->ref_count = 1 ;
 			}
 		}
+	}
 	return res ;
 }
 
@@ -547,9 +576,35 @@ forget_asimage( ASImage *im )
 			ASImageManager *imman = im->imageman ;
 			if( !AS_ASSERT(imman) )
 				remove_hash_item(imman->image_hash, (ASHashableValue)(char*)im->name, NULL, False);
-/*            im->ref_count = 0;     release_asimage should still correctly work !!!!
- *            im->imageman = NULL;
- */
+            im->ref_count = 0;
+            im->imageman = NULL;
+		}
+	}
+}
+
+void
+relocate_asimage( ASImageManager* to_imageman, ASImage *im )
+{
+	if( !AS_ASSERT(im) )
+	{
+		if( im->magic == MAGIC_ASIMAGE )
+		{
+			ASImageManager *imman = im->imageman ;
+			int ref_count = im->ref_count ; 
+			if( imman != NULL )
+			{
+				remove_hash_item(imman->image_hash, (ASHashableValue)(char*)im->name, NULL, False);
+	            im->ref_count = 0;
+    	        im->imageman = NULL;
+			}
+			if( to_imageman != NULL ) 
+			{
+				if( add_hash_item( to_imageman->image_hash, AS_HASHABLE(im->name), im) == ASH_Success ) 
+				{
+		            im->ref_count = ref_count < 1 ? 1: ref_count;
+    		        im->imageman = to_imageman ; 
+				}
+			}
 		}
 	}
 }
@@ -702,7 +757,7 @@ flip_gradient( ASGradient *orig, int flip )
 
 /* ******************** ASImageLayer ****************************/
 
-inline void
+void
 init_image_layers( register ASImageLayer *l, int count )
 {
 	memset( l, 0x00, sizeof(ASImageLayer)*count );
@@ -764,6 +819,8 @@ asimage_add_line_mono (ASImage * im, ColorPart color, CARD8 value, unsigned int
 	if (y >= im->height)
 		return 0;
 	
+	if( im->channels[color][y] ) 
+		forget_data( NULL, im->channels[color][y] ); 
 	im->channels[color][y] = store_data( NULL, &value, 1, 0, 0);
 	return im->width;
 }
@@ -815,7 +872,7 @@ void print_asimage( ASImage *im, int flags, char * func, int line )
 
 void print_component( register CARD32 *data, int nonsense, int len );
 
-inline int
+int
 asimage_decode_line (ASImage * im, ColorPart color, CARD32 * to_buf, unsigned int y, unsigned int skip, unsigned int out_width)
 {
 	ASStorageID id = im->channels[color][y];
@@ -825,13 +882,13 @@ LOCAL_DEBUG_CALLER_OUT( "im->width = %d, color = %d, y = %d, skip = %d, out_widt
 
 	if( id )
 	{
-		i = fetch_data32( NULL, id, to_buf, skip, out_width, 0);
+		i = fetch_data32( NULL, id, to_buf, skip, out_width, 0, NULL);
         LOCAL_DEBUG_OUT( "decoded %d pixels", i );
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
         {
             int z = -1 ;
             while( ++z < i )
-                fprintf( stderr, "%X ", to_buf[z] );
+                fprintf( stderr, "%lX ", to_buf[z] );
             fprintf( stderr, "\n");
 
         }
@@ -1027,6 +1084,67 @@ set_asimage_vector( ASImage *im, register double *vector )
 	return True;
 }
 
+ASVectorPalette*
+vectorize_asimage( ASImage *im, unsigned int max_colors, unsigned int dither,
+				   int opaque_threshold	)
+{
+	ASVectorPalette* pal ;
+	double *vec ;
+	ASColormap cmap;
+	int *res;
+    unsigned int r, g, b, v;
+	unsigned int x, y, j ;
+
+	if( im->alt.vector == NULL )
+		im->alt.vector = safemalloc( im->width*im->height*sizeof(double));
+	vec = im->alt.vector ;
+
+	/* contributed by Valeriy Onuchin from Root project at cern.ch */   
+
+ 	dither = dither > 7 ? 7 : dither;
+ 	res = colormap_asimage(im, &cmap, max_colors, dither, opaque_threshold);
+ 
+    for ( y = 0; y < im->height; y++) {
+       for ( x = 0; x < im->width; x++) {
+          int i = y*im->width + x;
+          g = INDEX_SHIFT_GREEN(cmap.entries[res[i]].green);
+          b = INDEX_SHIFT_BLUE(cmap.entries[res[i]].blue);
+          r = INDEX_SHIFT_RED(cmap.entries[res[i]].red);
+          v = MAKE_INDEXED_COLOR24(r,g,b);
+          v = (v>>12)&0x0FFF;
+          vec[(im->height - y - 1)*im->width + x] = ((double)v)/0x0FFF;
+       }
+    }
+
+    pal = safecalloc( 1, sizeof(ASVectorPalette));
+
+	pal->npoints = cmap.count ;	
+	pal->points = safemalloc( sizeof(double)*cmap.count);
+	pal->channels[IC_RED] = safemalloc( sizeof(CARD16)*cmap.count);
+	pal->channels[IC_GREEN] = safemalloc( sizeof(CARD16)*cmap.count);
+	pal->channels[IC_BLUE] = safemalloc( sizeof(CARD16)*cmap.count);
+	pal->channels[IC_ALPHA] = safemalloc( sizeof(CARD16)*cmap.count);
+ 
+    for ( j = 0; j < cmap.count; j++) {
+       g = INDEX_SHIFT_GREEN(cmap.entries[j].green);
+       b = INDEX_SHIFT_BLUE(cmap.entries[j].blue);
+       r = INDEX_SHIFT_RED(cmap.entries[j].red);
+       v = MAKE_INDEXED_COLOR24(r,g,b);
+ 
+       v = (v>>12)&0x0FFF;
+       pal->points[j] = ((double)v)/0x0FFF;
+ 		/* palette uses 16 bit color values for greater precision */
+       pal->channels[IC_RED][j] = cmap.entries[j].red<<QUANT_ERR_BITS;
+       pal->channels[IC_GREEN][j] = cmap.entries[j].green<<QUANT_ERR_BITS;
+       pal->channels[IC_BLUE][j] = cmap.entries[j].blue<<QUANT_ERR_BITS;
+       pal->channels[IC_ALPHA][j] = 0xFFFF;
+    }
+ 
+    destroy_colormap(&cmap, True);
+
+	return pal;
+}
+
 /* ********************************************************************************/
 /* Convinience function - very fast image cloning :                               */
 /* ********************************************************************************/
