@@ -101,7 +101,7 @@ asvisual_empty_XErrorHandler (Display * dpy, XErrorEvent * event)
     return 0;
 }
 /***************************************************************************/
-#ifdef LOCAL_DEBUG
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
 Status
 debug_AllocColor( const char *file, const char *func, int line, ASVisual *asv, Colormap cmap, XColor *pxcol )
 {
@@ -499,6 +499,8 @@ destroy_asvisual( ASVisual *asv, Bool reusable )
 		if( asv->glx_scratch_gc_indirect )
 			glXDestroyContext(dpy, asv->glx_scratch_gc_indirect );
 #endif
+		if( asv->scratch_window ) 
+			XDestroyWindow( asv->dpy, asv->scratch_window );
 
 #endif /*ifndef X_DISPLAY_MISSING */
 		if( !reusable )
@@ -1072,13 +1074,11 @@ create_visual_gc( ASVisual *asv, Window root, unsigned long mask, XGCValues *gcv
 #ifndef X_DISPLAY_MISSING
 	if( asv )
 	{
-		Window scratch_window = 0 ;
 		XGCValues scratch_gcv ;
-		if( (scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL )) != None )
-		{
-			gc = XCreateGC( asv->dpy, scratch_window, mask, gcvalues?gcvalues:&scratch_gcv );
-			XDestroyWindow( asv->dpy, scratch_window );
-		}
+		if( asv->scratch_window == None ) 
+			asv->scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL );
+		if( asv->scratch_window != None )
+			gc = XCreateGC( asv->dpy, asv->scratch_window, gcvalues?mask:0, gcvalues?gcvalues:&scratch_gcv );
 	}
 #endif
 	return gc;
@@ -1090,7 +1090,11 @@ create_visual_pixmap( ASVisual *asv, Window root, unsigned int width, unsigned i
 #ifndef X_DISPLAY_MISSING
 	Pixmap p = None ;
 	if( asv != NULL )
+	{	
+		if( root == None ) 
+			root = RootWindow(asv->dpy,DefaultScreen(asv->dpy));
 		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), (depth==0)?asv->true_depth:depth );
+	}
 	return p;
 #else
 	return None ;
@@ -1394,9 +1398,10 @@ int destroy_xshm_image( XImage *ximage )
 unsigned long 
 ximage2shmseg( XImage *xim )
 {
-	ASXShmImage *data = NULL ;
-	if( get_hash_item( xshmimage_images, AS_HASHABLE(xim), (void**)&data ) == ASH_Success )		
+	void *vptr = NULL ;
+	if( get_hash_item( xshmimage_images, AS_HASHABLE(xim), &vptr ) == ASH_Success )		
 	{
+		ASXShmImage *data = (ASXShmImage *)vptr ;
 		if( data->segment ) 
 			return data->segment->shmseg;	
 	}	
@@ -1443,14 +1448,14 @@ Bool ASPutXImage( ASVisual *asv, Drawable d, GC gc, XImage *xim,
 
 	if( ( img_data = check_XImage_shared( xim )) != NULL )
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim );
+/*		LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using shared memory Put = %p", xim ); */
 		if( XShmPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height, True ) )
 		{
 			img_data->wait_completion_event = True ;
 			return True ;
 		}
 	}
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim );
+/*	LOCAL_DEBUG_OUT( "XSHMIMAGE> PUT_XIM : using normal Put = %p", xim ); */
 	return XPutImage( asv->dpy, d, gc, xim, src_x, src_y, dest_x, dest_y,width, height );
 }
 
@@ -1806,6 +1811,7 @@ void pixel2color15bgr(ASVisual *asv, unsigned long pixel, CARD32 *red, CARD32 *g
 void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
+	register CARD32 *a = sl->alpha+sl->offset_x;
 	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
 	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
 	if( asv->msb_first )
@@ -1813,6 +1819,7 @@ void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regis
 		{
 			--i ;
 			{
+				a[i] = src[0];
 				r[i] = src[1];
 				g[i] = src[2];
 				b[i] = src[3];
@@ -1824,6 +1831,7 @@ void ximage2scanline32(ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regis
 		{
 			--i ;
 			{
+				a[i] = src[3];
 				r[i] = src[2];
 				g[i] = src[1];
 				b[i] = src[0];
@@ -1992,6 +2000,7 @@ ximage2scanline_pseudo12bpp( ASVisual *asv, XImage *xim, ASScanline *sl, int y,
 void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  register unsigned char *xim_data )
 {
 	register CARD32 *r = sl->xc1+sl->offset_x, *g = sl->xc2+sl->offset_x, *b = sl->xc3+sl->offset_x;
+	register CARD32 *a = sl->alpha+sl->offset_x;
 	register int i = MIN((unsigned int)(xim->width),sl->width-sl->offset_x);
 	register CARD8 *src = (CARD8*)(xim_data+(i-1)*4) ;
 	if( asv->msb_first )
@@ -1999,6 +2008,7 @@ void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 		{
 			--i ;
 			{
+				src[0] = a[i];
 				src[1] = r[i];
  				src[2] = g[i];
 				src[3] = b[i];
@@ -2010,6 +2020,7 @@ void scanline2ximage32( ASVisual *asv, XImage *xim, ASScanline *sl, int y,  regi
 		{
 			--i ;
 			{
+				src[3] = a[i];
 				src[2] = r[i];
 				src[1] = g[i];
 				src[0] = b[i];
