@@ -45,30 +45,31 @@ afterstep_parent_hints_func(Window parent, ASParentHints *dst )
 	dst->parent = parent ;
 	if( parent != Scr.Root )
 	{
-		ASWindow     *p;
+		ASWindow     *p = find_group_lead_aswindow( parent );
 
-        if ((p = window2ASWindow( parent )) == NULL) return False ;
-
-        dst->desktop = p->status->desktop ;
-		set_flags( dst->flags, AS_StartDesktop );
-
-		/* we may need to move our viewport so that the parent becomes visible : */
-        if ( !ASWIN_GET_FLAGS(p, AS_Iconic) )
+		if( p != NULL ) 
 		{
+	        dst->desktop = p->status->desktop ;
+			set_flags( dst->flags, AS_StartDesktop );
+
+			/* we may need to move our viewport so that the parent becomes visible : */
+    	    if ( !ASWIN_GET_FLAGS(p, AS_Iconic) )
+			{
 #if 0
-            int x, y ;
-            unsigned int width, height ;
-            x = p->status->x - p->decor->west ;
-            y = p->status->y - p->decor->north ;
-            width = p->status->width + p->decor->west + p->decor->east ;
-            height = p->status->height + p->decor->north + p->decor->south ;
+	            int x, y ;
+    	        unsigned int width, height ;
+        	    x = p->status->x - p->decor->west ;
+            	y = p->status->y - p->decor->north ;
+	            width = p->status->width + p->decor->west + p->decor->east ;
+    	        height = p->status->height + p->decor->north + p->decor->south ;
 
-            if( (dst->viewport_x = calculate_viewport( &x, width, p->scr->Vx, p->scr->MyDisplayWidth, p->scr->VxMax)) != p->scr->Vx )
-				set_flags( dst->flags, AS_StartViewportX );
+	            if( (dst->viewport_x = calculate_viewport( &x, width, p->scr->Vx, p->scr->MyDisplayWidth, p->scr->VxMax)) != p->scr->Vx )
+					set_flags( dst->flags, AS_StartViewportX );
 
-            if( (dst->viewport_y = calculate_viewport( &y, height, p->scr->Vy, p->scr->MyDisplayHeight, p->scr->VyMax)) != p->scr->Vy )
-				set_flags( dst->flags, AS_StartViewportY );
+	            if( (dst->viewport_y = calculate_viewport( &y, height, p->scr->Vy, p->scr->MyDisplayHeight, p->scr->VyMax)) != p->scr->Vy )
+					set_flags( dst->flags, AS_StartViewportY );
 #endif
+			}
 		}
 	}
 	return True ;
@@ -284,8 +285,7 @@ AddWindow (Window w, Bool from_map_request)
 	if( ASWIN_HFLAGS( tmp_win, AS_AvoidCover )  )
 		enforce_avoid_cover( tmp_win );
 
-
-	/* send_canvas_configure_notify(tmp_win->frame_canvas, tmp_win->client_canvas); */
+    /* SendConfigureNotify(tmp_win); */
 
     /*
 	 * Reparenting generates an UnmapNotify event, followed by a MapNotify.
@@ -438,6 +438,8 @@ LOCAL_DEBUG_CALLER_OUT( "asw(%p)->internal(%p)->data(%p)", asw, asw->internal, a
 		/* otherwise we can delete window normally - there are no recursion */
 	}
 
+	while( timer_remove_by_data( (void*)asw ) );
+	
     XSync (dpy, 0);
 	SendPacket (-1, M_DESTROY_WINDOW, 3, asw->w, asw->frame, (unsigned long)asw);
 
@@ -452,8 +454,11 @@ LOCAL_DEBUG_CALLER_OUT( "asw(%p)->internal(%p)->data(%p)", asw, asw->internal, a
         Scr.Windows->ungrabbed = NULL;
 
     if ( asw == Scr.Windows->active )
+	{
+		LOCAL_DEBUG_CALLER_OUT( "CHANGE Scr.Windows->active from %p to NULL", Scr.Windows->active );
         Scr.Windows->active = NULL;
-    if ( asw == Scr.Windows->hilited )
+    }
+	if ( asw == Scr.Windows->hilited )
         Scr.Windows->hilited = NULL;
     if ( asw == Scr.Windows->pressed )
         Scr.Windows->pressed = NULL;
