@@ -20,7 +20,9 @@
 #include "../../configure.h"
 #define LOCAL_DEBUG
 
+
 #include "asinternals.h"
+#include "../../libAfterConf/afterconf.h"
 
 #include <signal.h>
 #include <fcntl.h>
@@ -40,12 +42,13 @@
 ASFlagType AfterStepState = 0; /* default status */
 
 /* Config : */
-struct ASDatabase    *Database = NULL;
-
 ASVector     *Modules       = NULL;
 int           Module_fd     = 0;
 int           Module_npipes = 8;
 
+ASBalloonState *MenuBalloons = NULL ; 
+ASBalloonState *TitlebarBalloons = NULL ; 
+
 /**************************************************************************/
 void          SetupScreen();
 void          CleanupScreen();
@@ -72,6 +75,18 @@ LOCAL_DEBUG_OUT( "pid(%d),closing x_fd...", getpid() );
 LOCAL_DEBUG_OUT( "pid(%d),complete...", getpid() );
 }
 #endif
+#ifdef DEBUG_TRACE_X
+const char *window_id2name (Display * dpy, Window w, int *how_to_free)
+{
+	ASWindow *asw = window2ASWindow( w );
+	if( asw ) 
+	{
+		*how_to_free = 1 ;
+		return mystrdup(ASWIN_NAME(asw));
+	}
+	return NULL;
+}
+#endif
 
 
 /**************************************************************************/
@@ -81,7 +96,7 @@ LOCAL_DEBUG_OUT( "pid(%d),complete...", getpid() );
  *	main - start of afterstep
  ************************************************************************/
 int
-main (int argc, char **argv)
+main (int argc, char **argv, char **envp)
 {
     register int i ;
 	
@@ -113,6 +128,10 @@ main (int argc, char **argv)
 	trace_window_id2name_hook = &window_id2name;
 #endif
 	set_DeadPipe_handler(DeadPipe);
+
+#if !HAVE_DECL_ENVIRON
+	override_environ( envp );
+#endif
     InitMyApp( CLASS_AFTERSTEP, argc, argv, NULL, NULL, 0);
 	for( i = 1 ; i< argc ; ++i)
 	{
@@ -123,6 +142,8 @@ main (int argc, char **argv)
 	    }
 	}
 
+	LinkAfterStepConfig();
+
     AfterStepState = MyArgs.flags ;
     clear_flags( AfterStepState, ASS_NormalOperation );
 	set_flags( AfterStepState, ASS_SuppressDeskBack );
@@ -149,6 +170,8 @@ main (int argc, char **argv)
 		return 1;/* failed to accure window management selection - other wm is running */
 	}
 	XSetWindowBackground( dpy, Scr.Root, Scr.asv->black_pixel );
+	Scr.Look.desktop_animation_tint = get_random_tint_color();
+
     cover_desktop();
 	if( get_flags( AfterStepState, ASS_Restarting ))
 	{	
@@ -165,10 +188,6 @@ SHOW_CHECKPOINT;
 SHOW_CHECKPOINT;
 	XSync (dpy, 0);
 SHOW_CHECKPOINT;
-#ifdef XSHMIMAGE
-	enable_shmem_images ();
-SHOW_CHECKPOINT;
-#endif
     set_parent_hints_func( afterstep_parent_hints_func ); /* callback for collect_hints() */
 SHOW_CHECKPOINT;
     SetupModules();
@@ -251,7 +270,6 @@ SHOW_CHECKPOINT;
 
     /* Load config ... */
     /* read config file, set up menus, colors, fonts */
-    InitDatabase (False);
     LoadASConfig (0, PARSE_EVERYTHING);
 
     /* Reparent all the windows and setup pan frames : */
@@ -341,11 +359,12 @@ CreateManagementWindows()
 	LOCAL_DEBUG_OUT( "Scr.SizeWindow = %lX;", Scr.SizeWindow );
     /* create a window which will accept the keyboard focus when no other
 	   windows have it */
-    attr.event_mask = KeyPressMask | FocusChangeMask;
+    attr.event_mask = KeyPressMask | FocusChangeMask | AS_ROOT_EVENT_MASK;
     attr.override_redirect = True;
     Scr.ServiceWin = create_visual_window (Scr.asv, Scr.Root, 0, 0, 1, 1, 0,
                                            InputOutput, CWEventMask | CWOverrideRedirect,
                                            &attr);
+	set_service_window_prop (Scr.wmprops, Scr.ServiceWin);
 	LOCAL_DEBUG_OUT( "Scr.ServiceWin = %lX;", Scr.ServiceWin );
     XMapRaised (dpy, Scr.ServiceWin);
     XSetInputFocus (dpy, Scr.ServiceWin, RevertToParent, CurrentTime);
@@ -386,6 +405,9 @@ SetupScreen()
     Scr.Windows = init_aswindow_list();
 
 	XSelectInput (dpy, Scr.Root, AS_ROOT_EVENT_MASK);
+
+	MenuBalloons = create_balloon_state(); 
+	TitlebarBalloons = create_balloon_state(); 
 }
 
 void
@@ -399,6 +421,12 @@ CleanupScreen()
         destroy_aswindow_list( &(Scr.Windows), True );
         ungrab_server();
     }
+
+	destroy_balloon_state(&TitlebarBalloons); 
+	destroy_balloon_state(&MenuBalloons); 
+
+	release_all_old_background( True );
+
     DestroyManagementWindows();
     CleanupColormaps();
 
@@ -447,7 +475,10 @@ CleanupScreen()
         if( Scr.RootBackground->pmap )
         {
             if( Scr.wmprops->root_pixmap == Scr.RootBackground->pmap )
+			{	
                 set_xrootpmap_id (Scr.wmprops, None );
+				set_as_background(Scr.wmprops, None );
+			}
             XFreePixmap( dpy, Scr.RootBackground->pmap );
             ASSync(False);
             LOCAL_DEBUG_OUT( "root pixmap with id %lX destroyed", Scr.RootBackground->pmap );
@@ -524,7 +555,7 @@ CaptureAllWindows (ScreenInfo *scr)
     for (i = 0; i < nchildren; i++)
         if (children[i] )
         {
-            unsigned long nitems = 0;
+            long nitems = 0;
             CARD32 *state_prop = NULL ;
             int wm_state = DontCareState ;
             int k ;
@@ -556,7 +587,7 @@ CaptureAllWindows (ScreenInfo *scr)
             if( (wm_state == IconicState) || (attr.map_state != IsUnmapped))
 			{
 				LOCAL_DEBUG_OUT( "adding window %lX", children[i] );	  
-                AddWindow( children[i] );
+                AddWindow( children[i], False );
 			}
 		}
 
@@ -629,9 +660,19 @@ Done (Bool restart, char *command )
 		already_dead = True ;
 	}
 
+#ifdef XSHMIMAGE
+	/* may not need to do that as server may still have some of the shared 
+	 * memory and work in it */
+	flush_shm_cache();
+#endif
 
 LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?command:"");
 
+	XSelectInput( dpy, Scr.Root, 0 );	   
+	SendPacket (-1, M_SHUTDOWN, 0);
+	FlushAllQueues(); 
+	sleep_a_millisec(1000);
+
     /* lets duplicate the string so we don't accidental;y delete it while closing self down */
     if( restart )
 	{
@@ -674,12 +715,10 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
     CleanupScreen();
     /* Really make sure that the connection is closed and cleared! */
     XSync (dpy, 0);
+
 #ifdef XSHMIMAGE
-	/* may not need to do that as server may still have some of the shared 
-	 * memory and work in it */
 	flush_shm_cache();
 #endif
-
 	if (restart)
 	{
 		set_flags( MyArgs.flags, ASS_Restarting );
@@ -687,32 +726,47 @@ LOCAL_DEBUG_CALLER_OUT( "%s restart, cmd=\"%s\"", restart?"Do":"Don't", command?
                      None, C_NO_CONTEXT, False, restart_self, NULL );
     } else
 	{
+
 	    XCloseDisplay (dpy);
 		dpy = NULL ;
 
 		/* freeing up memory */
+		DestroyPendingFunctionsQueue();
+		DestroyCategories();
+
+	    cleanup_default_balloons();
+		destroy_asdatabase();
     	destroy_assession( Session );
 		destroy_asenvironment( &Environment );
-    	InitDatabase(True);
-    	free_func_hash ();
     	/* pixmap references */
     	build_xpm_colormap (NULL);
 
-#ifdef DEBUG_ALLOCS
         restack_window_list(INVALID_DESK, True);
+
         clientprops_cleanup ();
         wmprops_cleanup ();
+
         free_func_hash();
+		flush_keyword_ids();
         purge_asimage_registry();
-        flush_ashash_memory_pool();
-        flush_asbidirlist_memory_pool();
-        free( MyArgs.saved_argv );
+	
 		asxml_var_cleanup();
 		custom_color_cleanup(); 
+
+        free_as_app_args();
+		free( ASDefaultScr );
+
 		flush_default_asstorage();
+        flush_asbidirlist_memory_pool();
+        flush_ashash_memory_pool();
+#ifdef DEBUG_ALLOCS
         print_unfreed_mem ();
 #endif /*DEBUG_ALLOCS */
-    }
+#ifdef XSHMIMAGE
+		flush_shm_cache();
+#endif
+    
+	}
     exit(0);
 }
 
