@@ -69,6 +69,7 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module );
 void close_func_handler( FunctionData *data, ASEvent *event, int module );
 void restart_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_func_handler( FunctionData *data, ASEvent *event, int module );
+void desktop_entry_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_in_term_func_handler( FunctionData *data, ASEvent *event, int module );
 void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module );
 void change_background_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -96,6 +97,7 @@ void send_window_list_func_handler( FunctionData *data, ASEvent *event, int modu
 void save_workspace_func_handler( FunctionData *data, ASEvent *event, int module );
 void signal_reload_GTKRC_file_handler( FunctionData *data, ASEvent *event, int module );
 void KIPC_send_message_all_handler( FunctionData *data, ASEvent *event, int module );
+void set_func_handler( FunctionData *data, ASEvent *event, int module );
 void test_func_handler( FunctionData *data, ASEvent *event, int module );
 void screenshot_func_handler( FunctionData *data, ASEvent *event, int module );
 void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module );
@@ -149,6 +151,8 @@ void SetupFunctionHandlers()
 	function_handlers[F_CLOSE]          	= close_func_handler ;
     function_handlers[F_RESTART]            = restart_func_handler ;
 
+    function_handlers[F_DesktopEntry]		= desktop_entry_func_handler;
+
     function_handlers[F_EXEC] 				=
 		function_handlers[F_Swallow] 		=
 		function_handlers[F_MaxSwallow] 	= exec_func_handler ;
@@ -211,6 +215,7 @@ void SetupFunctionHandlers()
 
     function_handlers[F_QUICKRESTART]       = quickrestart_func_handler ;
     function_handlers[F_SEND_WINDOW_LIST]   = send_window_list_func_handler ;
+    function_handlers[F_SET]                = set_func_handler ;
     function_handlers[F_Test]               = test_func_handler ;
 
 	function_handlers[F_TAKE_WINDOWSHOT]    =
@@ -258,6 +263,7 @@ void destroy_scheduled_function_handler( void *data )
 }
 #define destroy_scheduled_function(sf)  destroy_scheduled_function_handler((void*)sf)
 
+static void DoExecuteFunction ( ASScheduledFunction *sf );
 
 /***********************************************************************
  *  Procedure:
@@ -301,7 +307,11 @@ ExecuteFunctionExt (FunctionData *data, ASEvent *event, int module, Bool defered
 	if( event->widget )
 	    sf->canvas              = event->widget->w ;
     }
-    append_bidirelem( FunctionQueue, sf );
+	/* we may end up deadlocked if we wait for modules who wait for window list : */
+	if( data->func == F_SEND_WINDOW_LIST ) 
+		DoExecuteFunction ( sf );
+	else
+	    append_bidirelem( FunctionQueue, sf );
 }
 
 void
@@ -510,6 +520,12 @@ void ExecutePendingFunctions()
 	    DoExecuteFunction( sf );
 }
 
+void DestroyPendingFunctionsQueue()
+{
+    if( FunctionQueue )
+		destroy_asbidirlist( &FunctionQueue );
+}
+
 /***********************************************************************
  *  Procedure:
  *	DeferExecution - defer the execution of a function to the
@@ -744,14 +760,9 @@ Bool desktop_category2complex_function( const char *name, const char *category_n
 		++(func->items_num);
 		init_func_data( fdata );
 		fdata->name = mystrdup("I");  /* for immidiate execution */ 
-		
-		if( get_flags( de->flags, ASDE_Terminal ) )
-			fdata->func = F_ExecInTerm;	   
-		else if( get_flags( de->flags, ASDE_ASModule ) )
-			fdata->func = F_MODULE;	   
-		else
-		 	fdata->func = F_EXEC;	
-        fdata->text = mystrdup( de->clean_exec );
+	 	fdata->func = desktop_entry2function_code( de );	
+		if( de->clean_exec ) 
+	        fdata->text = mystrdup( de->clean_exec );
 	}	
 
 	return True;	   
@@ -791,9 +802,17 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 	if( data->func == F_MOVE )
 	{
 	    if( data->func_val[0] != INVALID_POSITION )
-		x = new_val1;
+		{
+			x = new_val1;
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+				x -= asw->status->viewport_x ;
+		}
 	    if( data->func_val[1] != INVALID_POSITION )
-		y = new_val2;
+		{
+			y = new_val2;
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+				y -= asw->status->viewport_y ;
+		}
 	}else
 	{
 	    if( data->func_val[0] != INVALID_POSITION )
@@ -802,66 +821,64 @@ void moveresize_func_handler( FunctionData *data, ASEvent *event, int module )
 		height = new_val2;
 	}
 	moveresize_aswindow_wm( asw, x, y, width, height, False );
+
     }else
     {
-	ASMoveResizeData *mvrdata;
-	/*release_pressure(); */
-	if( data->func == F_MOVE )
-	    mvrdata = move_widget_interactively(Scr.RootCanvas,
-						asw->frame_canvas,
-						event,
-						apply_aswindow_move,
-						complete_aswindow_move );
-	else
-	{
-	    int side = 0 ;
-	    register unsigned long context = (event->context&C_FRAME);
+		ASMoveResizeData *mvrdata;
+		/*release_pressure(); */
+		if( data->func == F_MOVE )
+		{
+	    	mvrdata = move_widget_interactively(Scr.RootCanvas,
+							asw->frame_canvas,
+							event,
+							apply_aswindow_move,
+							complete_aswindow_move );
+		}else
+		{
+	    	int side = 0 ;
+	    	register unsigned long context = (event->context&C_FRAME);
 
-	    if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
-	    {
-		XBell (dpy, Scr.screen);
-		return;
-	    }
+	    	if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
+	    	{
+			XBell (dpy, Scr.screen);
+			return;
+	    	}
 
-	    while( (0x01&context) == 0 && side <= FR_SE)
-	    {
-		++side ;
-		context = context>>1 ;
-	    }
+	    	while( (0x01&context) == 0 && side <= FR_SE)
+	    	{
+			++side ;
+			context = context>>1 ;
+	    	}
 
-			if( side > FR_SE )
-			{
-				int pointer_x = 0, pointer_y = 0 ;
-				ASQueryPointerRootXY( &pointer_x, &pointer_y );
-				if( pointer_x > asw->frame_canvas->root_x + asw->frame_canvas->width/2 )
+				if( side > FR_SE )
 				{
-					if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
-						side = FR_SE ;
+					int pointer_x = 0, pointer_y = 0 ;
+					ASQueryPointerRootXY( &pointer_x, &pointer_y );
+					if( pointer_x > asw->frame_canvas->root_x + asw->frame_canvas->width/2 )
+					{
+						if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
+							side = FR_SE ;
+						else
+							side = FR_NE ;
+					}else if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
+						side = FR_SW ;
 					else
-						side = FR_NE ;
-				}else if( pointer_y > asw->frame_canvas->root_y + asw->frame_canvas->height/2 )
-					side = FR_SW ;
-				else
-					side = FR_NW ;
-			}
+						side = FR_NW ;
+				}
 
-	    mvrdata = resize_widget_interactively(  Scr.RootCanvas,
-						    asw->frame_canvas,
-						    event,
-						    apply_aswindow_moveresize,
-						    complete_aswindow_moveresize,
-						    side );
-	}
-	if( mvrdata )
-	{
+	    	mvrdata = resize_widget_interactively(  Scr.RootCanvas,
+						    	asw->frame_canvas,
+						    	event,
+						    	apply_aswindow_moveresize,
+						    	complete_aswindow_moveresize,
+						    	side );
+		}
+		if( mvrdata )
+		{
 			mvrdata->move_only = (data->func == F_MOVE) ;
-	    raise_scren_panframes( ASDefaultScr );
-	    mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
-	    set_moveresize_restrains( mvrdata, asw->hints, asw->status);
-	    mvrdata->grid = make_desktop_grid(Scr.CurrentDesk, AS_LayerDesktop, False, Scr.Vx, Scr.Vy, asw);
-	    Scr.moveresize_in_progress = mvrdata ;
-	    ASWIN_SET_FLAGS( asw, AS_MoveresizeInProgress );
-	}
+			setup_aswindow_moveresize(asw, mvrdata );
+	    	ASWIN_SET_FLAGS( asw, AS_MoveresizeInProgress );
+		}
     }
 }
 
@@ -1012,7 +1029,7 @@ void setlayer_func_handler( FunctionData *data, ASEvent *event, int module )
 void change_desk_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     if( event->client )
-	change_aswindow_desktop( event->client, data->func_val[0], False );
+		change_aswindow_desktop( event->client, data->func_val[0], False );
 }
 
 void toggle_status_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1085,11 +1102,13 @@ void pin_menu_func_handler( FunctionData *data, ASEvent *event, int module )
 {
     ASMenu *menu = NULL;
 	char * menu_name = data->text? data->text:data->name ;
-    if( menu_name )
+	LOCAL_DEBUG_OUT( "menu_name = \"%s\", client = %p, internal = %p", menu_name?"NULL":menu_name, event->client, event->client->internal );
+    if( menu_name && menu_name[0] != '\0' )
 		menu = find_asmenu( menu_name );
     else if( event->client && event->client->internal )
     {
 		ASMagic *data = event->client->internal->data ;
+		LOCAL_DEBUG_OUT( "data = %p, magic = %lX", data, data->magic );
 		if( data->magic == MAGIC_ASMENU )
 	    	menu = (ASMenu*)data;
     }
@@ -1276,6 +1295,23 @@ void exec_tool_func_handler( FunctionData *data, ASEvent *event, int module )
 	}
 }
 
+void desktop_entry_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	ASDesktopEntry *de = name2desktop_entry( data->text?data->text:data->name, NULL );
+	if( de == NULL ) 
+		XBell (dpy, event->scr->screen);
+	else
+	{
+		FunctionData *real_data = desktop_entry2function( de, NULL);
+		if( real_data ) 
+		{
+			ExecuteFunctionExt( real_data, event, -1, True );
+			destroy_func_data( &real_data );
+		}
+	}	
+}
+
+
 static int _as_config_change_recursion = 0 ;
 static int _as_config_change_count = 0 ;
 static int _as_background_change_count = 0 ;
@@ -1652,6 +1688,8 @@ void refresh_func_handler( FunctionData *data, ASEvent *event, int module )
     XSync (dpy, False);
     XDestroyWindow (dpy, w);
     XFlush (dpy);
+	
+	spool_unfreed_mem( "afterstep.allocs.refresh", NULL );
 }
 
 void goto_page_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1696,12 +1734,100 @@ void gethelp_func_handler( FunctionData *data, ASEvent *event, int module )
 
 void wait_func_handler( FunctionData *data, ASEvent *event, int module )
 {
+	ASWindow *asw ;
 	char *complex_pattern = data->text ;
 	if( data->name && data->name[1] == ':' )
 		complex_pattern = &(data->name[2]);
 
-	WaitWindowLoop( complex_pattern, -1 );
-	LOCAL_DEBUG_OUT( "Wait completed for \"%s\"", complex_pattern );
+	asw = WaitWindowLoop( complex_pattern, -1 );
+	LOCAL_DEBUG_OUT( "Wait completed for \"%s\", asw = %p, data->text = \"%s\"", complex_pattern, asw, data->text?data->text:"(null)" );
+	if( asw && data->text ) 
+	{
+		/* 1. parse text into a name_list struct */	
+		name_list *style = string2DatabaseStyle (data->text);
+		LOCAL_DEBUG_OUT( "style = %p, set_data_flags = 0x%lX, set_flags = 0x%lX", style, style?style->set_data_flags:0, style?style->set_flags:0 );
+		if( style ) 
+		{	/* 2. apply set data from name_list to asw */	
+			int new_vx = Scr.Vx ;
+			int	new_vy = Scr.Vy ;
+			int x, y ; 
+			int width, height ;
+			ASFlagType geom_flags = get_flags (style->set_data_flags, STYLE_DEFAULT_GEOMETRY)?style->default_geometry.flags:0;
+			ASFlagType state_flags = 0 ;
+
+			if( get_flags (style->set_data_flags, STYLE_VIEWPORTX) )
+				new_vx = style->ViewportX ;
+			else
+				new_vx = asw->status->viewport_x ;
+			if( get_flags (style->set_data_flags, STYLE_VIEWPORTY) )
+				new_vy = style->ViewportY ;
+			else
+				new_vy = asw->status->viewport_y ;
+			
+			x = get_flags( geom_flags, XValue )?style->default_geometry.x:asw->status->x ; 
+			y = get_flags( geom_flags, YValue )?style->default_geometry.y:asw->status->y ; 
+			width = get_flags( geom_flags, WidthValue )?
+						(style->default_geometry.width+asw->status->frame_size[FR_W]+asw->status->frame_size[FR_E]):
+						asw->status->width ; 
+			height = get_flags( geom_flags, HeightValue )?
+						(style->default_geometry.height+asw->status->frame_size[FR_N]+asw->status->frame_size[FR_S]):
+						asw->status->height ; 
+			
+			if( get_flags (style->set_flags, STYLE_STICKY))
+			{
+				if( (get_flags (style->flags, STYLE_STICKY) && !ASWIN_GET_FLAGS(asw,AS_Sticky)) ||
+					(!get_flags (style->flags, STYLE_STICKY) && ASWIN_GET_FLAGS(asw,AS_Sticky)) )
+					set_flags( state_flags, AS_Sticky );
+			}
+			if( get_flags (style->set_flags, STYLE_FULLSCREEN))
+			{
+				if( (get_flags (style->flags, STYLE_FULLSCREEN) && !ASWIN_GET_FLAGS(asw,AS_Fullscreen)) ||
+					(!get_flags (style->flags, STYLE_FULLSCREEN) && ASWIN_GET_FLAGS(asw,AS_Fullscreen)) )
+					set_flags( state_flags, AS_Fullscreen );
+			}
+
+			if( state_flags != 0 ) 
+				toggle_aswindow_status( asw, state_flags );
+
+			if( !ASWIN_GET_FLAGS(asw,AS_Sticky) )
+			{	
+			 	if(	get_flags (style->set_data_flags, STYLE_STARTUP_DESK) )
+						change_aswindow_desktop( asw, style->Desk, False );
+			}else
+			{
+				new_vx = Scr.Vx ;	
+				new_vy = Scr.Vy ;
+			}	 
+			
+			LOCAL_DEBUG_OUT( "%s -> viewport : new(%+d%+d)old(%+d%+d) ; geom : new(%dx%d%+d%+d)old(%dx%d%+d%+d)", 
+							 ASWIN_GET_FLAGS(asw,AS_Sticky)?"sticky":"slippery", 
+							 new_vx, new_vy, asw->status->viewport_x, asw->status->viewport_y,
+							 width, height, x, y, asw->status->width, asw->status->height, asw->status->x, asw->status->y );
+			
+							   
+			if( get_flags (style->set_data_flags, STYLE_DEFAULT_GEOMETRY|STYLE_VIEWPORTX|STYLE_VIEWPORTY) )
+			{
+				x += Scr.Vx - asw->status->viewport_x + new_vx ;
+				y += Scr.Vy - asw->status->viewport_y + new_vy ;
+				moveresize_aswindow_wm( asw, x, y, width, height, False );
+			}	 
+			
+			if( get_flags (style->set_data_flags, STYLE_LAYER) && ASWIN_LAYER(asw) != style->layer )
+			{
+				LOCAL_DEBUG_OUT( "new_layer = %d", 	  style->layer );
+				change_aswindow_layer( asw, style->layer );
+			}
+							 
+			if( get_flags (style->set_flags, STYLE_START_ICONIC ) )
+			{
+				if ( (get_flags (style->flags, STYLE_START_ICONIC ) && !ASWIN_GET_FLAGS(asw, AS_Iconic))||  
+					 (!get_flags (style->flags, STYLE_START_ICONIC ) && ASWIN_GET_FLAGS(asw, AS_Iconic)))
+					set_window_wm_state( event->client, get_flags (style->flags, STYLE_START_ICONIC ), False );
+			}	 
+			
+			style_delete (style, NULL);
+		}	
+	}	 
 	XSync (dpy, 0);
 }
 
@@ -1765,12 +1891,11 @@ void restartmodule_func_handler( FunctionData *data, ASEvent *event, int module
 {
 	char *cmd_line = GetModuleCmdLineByName(data->text);
 	KillModuleByName (data->text);	
-	if( cmd_line ) 
-	{
-	    UngrabEm ();
-    	ExecModule (cmd_line, event->client ? event->client->w : None, event->context);
+	
+    UngrabEm ();
+   	ExecModule (cmd_line?cmd_line:data->text, event->client ? event->client->w : None, event->context);
+	if( cmd_line )
 		free( cmd_line );
-	}	 
 }
 
 void popup_func_handler( FunctionData *data, ASEvent *event, int module )
@@ -1789,8 +1914,14 @@ void windowlist_func_handler( FunctionData *data, ASEvent *event, int module )
     MenuData *md =  make_desk_winlist_menu( Scr.Windows, data->text == NULL ? event->scr->CurrentDesk: data->func_val[0], Scr.Feel.winlist_sort_order, False );
     if( md != NULL )
 	{	
-		run_menu_data( md );
-		destroy_menu_data( &md );
+		ASMenu *menu = run_menu_data( md );
+			/* attaching menu data to menu, so that we can destroy it when menu closes.
+			   This is to accomodate those dynamically generated menus, 
+			   such as window list and module list */
+		   	/* Crude! should implement reference counting instead  */
+		
+		if( menu ) 
+			menu->volitile_menu_data = md ; 
 	}	
 #endif /* ! NO_WINDOWLIST */
 }
@@ -1802,8 +1933,9 @@ void modulelist_func_handler( FunctionData *data, ASEvent *event, int module )
 						make_restart_module_menu(Scr.Feel.winlist_sort_order );
     if( md != NULL )
 	{	
-		run_menu_data( md );
-		destroy_menu_data( &md );
+		ASMenu *menu = run_menu_data( md );
+		if( menu ) 
+			menu->volitile_menu_data = md ; 
 	}	 
 
 }
@@ -1823,10 +1955,10 @@ send_aswindow_data_iter_func(void *data, void *aux_data)
 
     SendConfig (module_id.id, M_CONFIGURE_WINDOW, asw);
 	/* always start with RES_CLASS to let module know if this is a DockApp or not */
-    SendString (module_id.id, M_RES_CLASS, asw->w, asw->frame, asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
-    SendString (module_id.id, M_RES_NAME,  asw->w, asw->frame, asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
-    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, asw->hints->names_encoding[asw->hints->icon_name_idx]);
-    SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], asw->hints->names_encoding[0]);
+    SendString (module_id.id, M_RES_CLASS, asw->w, asw->frame, asw, asw->hints->res_class, get_hint_name_encoding(asw->hints, asw->hints->res_class_idx));
+    SendString (module_id.id, M_RES_NAME,  asw->w, asw->frame, asw, asw->hints->res_name, get_hint_name_encoding(asw->hints, asw->hints->res_name_idx));
+    SendString (module_id.id, M_ICON_NAME, asw->w, asw->frame, asw, asw->hints->icon_name, get_hint_name_encoding(asw->hints, asw->hints->icon_name_idx));
+	SendString (module_id.id, M_WINDOW_NAME, asw->w,asw->frame, asw, asw->hints->names[0], get_hint_name_encoding(asw->hints, 0));
     return True;
 }
 
@@ -1907,6 +2039,54 @@ void swallow_window_func_handler( FunctionData *data, ASEvent *event, int module
  	}	
 }	 
 
+void set_func_handler( FunctionData *data, ASEvent *event, int module )
+{
+	if( data->text != NULL ) 
+	{
+		char *eq_ptr = strchr( data->text,'=' ); 
+		if( eq_ptr ) 
+		{
+			int val  = 0 ;
+			char *tail = eq_ptr+1; 
+			val = (int)parse_math(tail, &tail, strlen(tail));
+			LOCAL_DEBUG_OUT( "tail = \"%s\", val = %d", tail, val );
+			if( tail > eq_ptr+1 ) 
+			{
+				*eq_ptr = '\0' ; 
+				asxml_var_insert(data->text, val);
+				if( strncmp( data->text, "menu.", 5 ) == 0 )
+				{
+					ASFlagType change_flag = 0 ; 
+					ASFlagType old_look_flags = Scr.Look.flags ; 
+					if( strcmp( data->text, ASXMLVAR_MenuShowMinipixmaps ) == 0 )
+						change_flag = MenuMiniPixmaps ;
+					else if( strcmp( data->text, ASXMLVAR_MenuShowUnavailable ) == 0 )
+						change_flag = MenuShowUnavailable ;
+					else if( strcmp( data->text, ASXMLVAR_MenuTxtItemsInd ) == 0 )
+						change_flag = TxtrMenuItmInd ;
+					if( change_flag != 0 ) 
+					{
+						if( !val ) 
+							clear_flags( Scr.Look.flags, change_flag );
+						else 
+							set_flags( Scr.Look.flags, change_flag );
+					}else if( strcmp( data->text, ASXMLVAR_MenuRecentSubmenuItems) == 0 )
+					{
+						Scr.Feel.recent_submenu_items = (val <= 0)?0:val ;
+					}
+					
+					if( old_look_flags != Scr.Look.flags ) 
+					{
+						/* need to referesh menus maybe? */
+					}
+					LOCAL_DEBUG_OUT( "old_flags = %lX, flags = %lX", old_look_flags, Scr.Look.flags );
+				}
+				*eq_ptr = '=' ; 
+			}
+		}
+	}
+}
+
 void test_func_handler( FunctionData *data, ASEvent *event, int module )
 {
 	 /* add test command processing here : */
