@@ -121,15 +121,16 @@ LOCAL_DEBUG_OUT( "module name \"%s\"", module->name );
     	while (module->output_queue != NULL)
         	DeleteQueueBuff (module);
     	if (module->name != NULL)
-        	free (module->name);
-    	if (module->ibuf.text != NULL)
-        	free (module->ibuf.text);
+        destroy_string( &(module->name));
+        destroy_string( &(module->cmd_line));
+    	destroy_string( &(module->ibuf.text));
 
     	if (module->ibuf.func != NULL)
 		{
         	free_func_data (module->ibuf.func);
         	free (module->ibuf.func);
 		}
+		
 
     	memset( module, 0x00, sizeof(module_t) );
 	}else
@@ -554,7 +555,7 @@ int
 AcceptModuleConnection (int socket_fd)
 {
 	int           fd;
-	int           len = sizeof (struct sockaddr_un);
+	unsigned int  len = sizeof (struct sockaddr_un);
 	struct sockaddr_un name;
 
 	fd = accept (socket_fd, (struct sockaddr *)&name, &len);
@@ -711,7 +712,7 @@ DeadPipe (int nonsense)
 int FindModuleByName (char *name)
 {
 	int module = -1;
-	if( Modules ) 
+	if( Modules && name) 
 	{	
     	wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
 
@@ -745,8 +746,6 @@ GetModuleCmdLineByName(char *name)
 		register module_t *list = MODULES_LIST ;	
 		if( list[module].cmd_line ) 
 			return mystrdup( list[module].cmd_line ) ;
-		else
-			return mystrdup( list[module].name );
 	}
 	return NULL ;
 }
@@ -839,10 +838,10 @@ SendConfig (int module, send_data_type event_type, ASWindow * t)
     }
 
 	asw_id.asw = t ;
-    SendPacket (module, event_type, 25,
+    SendPacket (module, event_type, 26,
                 t->w,                 t->frame,              asw_id.id,
                 frame_x,              frame_y,               frame_width,         frame_height,
-                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,
+                ASWIN_DESK(t),        t->status->flags,      t->hints->flags,     t->hints->client_icon_flags,
                 t->hints->base_width, t->hints->base_height, t->hints->width_inc, t->hints->height_inc,
                 t->hints->min_width,  t->hints->min_height,  t->hints->max_width,  t->hints->max_height,
                 t->hints->gravity,    icon_title_w,          icon_pixmap_w,
@@ -1038,7 +1037,7 @@ broadcast_window_name( ASWindow *asw )
     if( asw )
 	{	
         SendString( -1, M_WINDOW_NAME, asw->w, asw->frame,
-                    asw, ASWIN_NAME(asw), asw->hints->names_encoding[0]);
+                    asw, ASWIN_NAME(asw), get_hint_name_encoding(asw->hints, 0));
         SendString( -1, M_WINDOW_NAME_MATCHED, asw->w, asw->frame,
                     asw, asw->hints->matched_name0, asw->hints->matched_name0_encoding );
 	}
@@ -1048,8 +1047,10 @@ void
 broadcast_icon_name( ASWindow *asw )
 {
     if( asw )
+	{
         SendString( -1, M_ICON_NAME, asw->w, asw->frame,
-                    asw, ASWIN_ICON_NAME(asw), asw->hints->names_encoding[asw->hints->icon_name_idx]);
+                    asw, ASWIN_ICON_NAME(asw), get_hint_name_encoding(asw->hints, asw->hints->icon_name_idx));
+	}
 }
 
 void
@@ -1058,9 +1059,9 @@ broadcast_res_names( ASWindow *asw )
     if( asw )
     {
         SendString( -1, M_RES_CLASS, asw->w, asw->frame,
-                    asw, asw->hints->res_class, asw->hints->names_encoding[asw->hints->res_class_idx]);
+                    asw, asw->hints->res_class, get_hint_name_encoding(asw->hints, asw->hints->res_class_idx));
         SendString( -1, M_RES_NAME, asw->w, asw->frame,
-                    asw, asw->hints->res_name, asw->hints->names_encoding[asw->hints->res_name_idx]);
+                    asw, asw->hints->res_name, get_hint_name_encoding(asw->hints, asw->hints->res_name_idx));
     }
 }
 
@@ -1142,7 +1143,20 @@ make_module_menu( FunctionCode func, const char *title, int sort_order )
 				minipixmap = de->fulliconname ;
 
 			if( (mdi = add_menu_fdata_item( md, menuitems[i], minipixmap, NULL)) != NULL ) 
+			{
 				set_flags( mdi->flags, MD_ScaleMinipixmapDown );
+				if( de )
+				{
+					char *comment = NULL; 
+					if( dup_desktop_entry_Comment( de, &comment ) )
+						set_flags( mdi->flags, MD_CommentIsUTF8 );
+					if( comment )
+					{
+						mdi->comment = interpret_ascii_string( comment );
+						free( comment );
+					}
+				}
+			}
 			safefree( menuitems[i] ); /* scrubba-dub-dub */
 		}
 		safefree(menuitems);
