@@ -1,4 +1,6 @@
 /*
+ * Copyright (C) 2005 Fabian Yamaguchi
+ * Copyright (C) 2004-2005 Sasha Vasko
  * Copyright (C) 1996 Frank Fejes
  * Copyright (C) 1995 Bo Yang
  * Copyright (C) 1993 Robert Nation
@@ -58,85 +60,6 @@ typedef struct ASAvoidCoverAuxData
  * space each :
  */
 /*************************************************************************/
-void
-subtract_rectangle_from_list( ASVector *list, int left, int top, int right, int bottom )
-{
-    register int i = PVECTOR_USED(list);
-    XRectangle *rects = PVECTOR_HEAD(XRectangle,list);
-    XRectangle tmp ;
-    /* must trace in reverse order ! */
-    while( --i >= 0 )
-    {
-        int r_left = rects[i].x, r_right = rects[i].x+rects[i].width ;
-        int r_top = rects[i].y, r_bottom = rects[i].y+rects[i].height ;
-        Bool disected = False ;
-        /* we can build at most 4 rectangles from each substraction : */
-        if( top < r_bottom && bottom >= r_top )
-        {   /* we may need to create 2 vertical rectangles ( left and right ) :*/
-            /* left rectangle : */
-            tmp.y = r_top ;
-            tmp.height = r_bottom - r_top ;
-             if( left > r_left && left < r_right )
-            {
-                rects[i].x = r_left ;
-                rects[i].width = left - r_left ;
-                /* y and height remain unchanged ! */
-                disected = True ;
-            }
-            /* right rectangle : */
-            if( right > r_left && right < r_right )
-            {
-                tmp.x = right ;
-                tmp.width = r_right - right ;
-                if( disected )
-                {
-                    append_vector( list, &tmp, 1 );
-                    rects = PVECTOR_HEAD(XRectangle,list); /* memory may have gotten reallocated */
-                }else
-                {
-                    rects[i] = tmp ;
-                    disected = True ;
-                }
-            }
-        }
-        if( left < r_right && right >= r_left )
-        {   /* we may need to create 2 horizontal rectangles ( top and bottom ) :*/
-            /* top rectangle : */
-            tmp.x = r_left ;
-            tmp.width = r_right - r_left ;
-            if( top > r_top && top < r_bottom )
-            {
-                tmp.y = r_top ;
-                tmp.height = top- r_top ;
-                if( disected )
-                {
-                    append_vector( list, &tmp, 1 );
-                    rects = PVECTOR_HEAD(XRectangle,list); /* memory may have gotten reallocated */
-                }else
-                {
-                    rects[i] = tmp ;
-                    disected = True ;
-                }
-            }
-            /* bottom rectangle */
-            if( bottom > r_top && bottom < r_bottom )
-            {
-                tmp.y = bottom ;
-                tmp.height = r_bottom- bottom ;
-                if( disected )
-                {
-                    append_vector( list, &tmp, 1 );
-                    rects = PVECTOR_HEAD(XRectangle,list); /* memory may have gotten reallocated */
-                }else
-                {
-                    rects[i] = tmp ;
-                    disected = True ;
-                }
-            }
-        }
-    }
-}
-
 
 Bool
 get_free_rectangles_iter_func(void *data, void *aux_data)
@@ -151,7 +74,10 @@ get_free_rectangles_iter_func(void *data, void *aux_data)
     {
         int x, y;
         unsigned int width, height, bw;
-        if( !ASWIN_HFLAGS(asw, AS_AvoidCover) && fr_data->min_layer > ASWIN_LAYER(asw))
+        if( ASWIN_GET_FLAGS(asw, AS_Fullscreen|AS_ShortLived ) )
+			return True;
+
+		if( !ASWIN_HFLAGS(asw, AS_AvoidCover) && fr_data->min_layer > ASWIN_LAYER(asw))
             return True;
 
         if( ASWIN_GET_FLAGS(asw, AS_Iconic ) )
@@ -169,31 +95,15 @@ get_free_rectangles_iter_func(void *data, void *aux_data)
             get_current_canvas_geometry( asw->frame_canvas, &x, &y, &width, &height,&bw );
         x += Scr.Vx ;
         y += Scr.Vy ;
-        if( x+width+bw >= min_vx && x-bw < max_vx &&  y+height+bw >= min_vy && y-bw < max_vy )
-            subtract_rectangle_from_list( fr_data->list, x-bw, y-bw, x+width+bw, y+height+bw );
+		LOCAL_DEBUG_OUT( "frame_geom = %dx%d%+d%+d, h_limits = %d,%d; v_limits = %d,%d", 
+						  width, height, x-bw, y-bw, min_vx, max_vx, min_vy, max_vy );
+        if( x+(int)width+(int)bw >= min_vx && x-(int)bw < max_vx &&  y+(int)height+(int)bw >= min_vy && y-(int)bw < max_vy )
+            subtract_rectangle_from_list( fr_data->list, x-(int)bw, y-(int)bw, x+(int)width+(int)bw, y+(int)height+(int)bw );
     }
 
     return True;
 }
 
-#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
-static void
-print_rectangles_list( ASVector *list )
-{
-    XRectangle *rects = PVECTOR_HEAD(XRectangle,list);
-    int i = PVECTOR_USED(list);
-
-    fprintf( stderr, "\tRectangles.count = %d;\n", i );
-    while ( --i >= 0 )
-    {
-        fprintf( stderr, "\tRectangles[%d].x = %d;\n", i, rects[i].x );
-        fprintf( stderr, "\tRectangles[%d].y = %d;\n", i, rects[i].y );
-        fprintf( stderr, "\tRectangles[%d].width = %d;\n", i, rects[i].width );
-        fprintf( stderr, "\tRectangles[%d].height = %d;\n", i, rects[i].height );
-    }
-}
-#endif
-
 static ASVector *
 build_free_space_list( ASWindow *to_skip, ASGeometry *area, int min_layer )
 {
@@ -303,9 +213,9 @@ make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWi
 //    add_canvas_grid( grid_data.grid, Scr.RootCanvas, resist, attract, vx, vy );
 
 	add_gridline( &(grid_data.grid->h_lines), 0,                   0, Scr.MyDisplayWidth,  resist, attract );
-    add_gridline( &(grid_data.grid->h_lines), Scr.MyDisplayHeight, 0, Scr.MyDisplayWidth,  resist, attract );
+    add_gridline( &(grid_data.grid->h_lines), Scr.MyDisplayHeight, 0, Scr.MyDisplayWidth,  attract, resist );
     add_gridline( &(grid_data.grid->v_lines), 0,                   0, Scr.MyDisplayHeight, resist, attract );
-    add_gridline( &(grid_data.grid->v_lines), Scr.MyDisplayWidth , 0, Scr.MyDisplayHeight, resist, attract );
+    add_gridline( &(grid_data.grid->v_lines), Scr.MyDisplayWidth , 0, Scr.MyDisplayHeight, attract, resist );
 
     /* add all the window edges for this desktop : */
     iterate_asbidirlist( Scr.Windows->clients, get_aswindow_grid_iter_func, (void*)&grid_data, NULL, False );
@@ -317,6 +227,25 @@ make_desktop_grid(int desk, int min_layer, Bool frame_only, int vx, int vy, ASWi
     return grid_data.grid;
 }
 
+void setup_aswindow_moveresize(ASWindow *asw,  struct ASMoveResizeData *mvrdata )
+{
+	if( asw->frame_data && asw->tbar )
+	{
+		if( asw->frame_data->condense_titlebar != NO_ALIGN /* && mvrdata->move_only  */)  
+		{
+			if( ASWIN_HFLAGS(asw, AS_VerticalTitle ) )
+				mvrdata->title_west = asw->tbar->width ;
+			else
+				mvrdata->title_north = asw->tbar->height ;
+		}
+	}
+	raise_scren_panframes( ASDefaultScr );
+	mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
+	set_moveresize_restrains( mvrdata, asw->hints, asw->status);
+	mvrdata->grid = make_desktop_grid(Scr.CurrentDesk, AS_LayerDesktop, False, Scr.Vx, Scr.Vy, asw);
+	Scr.moveresize_in_progress = mvrdata ;
+}
+
 void apply_aswindow_moveresize(struct ASMoveResizeData *data)
 {
     ASWindow *asw = window2ASWindow( AS_WIDGET_WINDOW(data->mr));
@@ -325,15 +254,58 @@ LOCAL_DEBUG_OUT( "%dx%d%+d%+d", data->curr.width, data->curr.height, data->curr.
     {
 		int new_width = data->curr.width ;
 		int new_height = data->curr.height ;
+		Bool server_grabbed = False ; 
         if( ASWIN_GET_FLAGS( asw, AS_Shaded ) )
 		{
 			new_width = asw->status->width ;
 			new_height = asw->status->height ;
+#if 0
+			/* lets only move us as we are in shaded state : */
+	        move_canvas(  asw->frame_canvas, data->curr.x, data->curr.y );
+		}else
+		{
+			if( data->curr.width != data->last.width || 
+			    data->curr.height != data->last.height )
+			{
+				int client_width = data->curr.width - (asw->frame_canvas->width - asw->client_canvas->width) ; 
+				int client_height = data->curr.height - (asw->frame_canvas->height - asw->client_canvas->height) ; 
+				XGrabServer(dpy);
+				server_grabbed = True ;
+				resize_canvas( asw->client_canvas, client_width, client_height);
+			}
+			moveresize_canvas(  asw->frame_canvas, data->curr.x, data->curr.y, data->curr.width, data->curr.height );
+			ASSync(False);
+#endif		
+		}
+		if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor|AS_Shaped ) && 
+			(data->curr.width > data->last.width || data->curr.height > data->last.height) )
+		{/* this greately reduces flickering on resizing of shaped windows : */
+		    XRectangle    rect;
+    		rect.x = 0;
+    		rect.y = 0;
+			moveresize_canvas(  asw->frame_canvas, data->curr.x, data->curr.y, data->curr.width, data->curr.height );
+			if( data->curr.width > data->last.width ) 
+			{
+				if( get_flags(asw->frame_data->condense_titlebar, ALIGN_LEFT) && asw->tbar )
+					rect.y = asw->tbar->height ; 
+	    		rect.width  = data->curr.width - data->last.width;
+    			rect.height = (int)data->last.height - rect.y;
+				XShapeCombineRectangles ( dpy, asw->frame, ShapeBounding,
+                                		  data->last.width, 0, &rect, 1, ShapeUnion, Unsorted);
+			}
+			if( data->curr.height > data->last.height ) 
+			{
+				if( get_flags(asw->frame_data->condense_titlebar, ALIGN_RIGHT) && asw->tbar )
+					rect.x = asw->tbar->width ; 
+	    		rect.width  = (int)data->curr.width - rect.x;
+    			rect.height = data->curr.height - data->last.height;
+				XShapeCombineRectangles ( dpy, asw->frame, ShapeBounding,
+                                		  0, data->last.height, &rect, 1, ShapeUnion, Unsorted);
+			}
 		}
-		/* lets only move us as we maybe in shaded state : */
-        move_canvas(  asw->frame_canvas, data->curr.x, data->curr.y );
-		ASSync(False);
         moveresize_aswindow_wm( asw, data->curr.x, data->curr.y, new_width, new_height, False);
+		if( server_grabbed ) 
+			XUngrabServer(dpy);
     }
 }
 
@@ -373,7 +345,6 @@ void complete_aswindow_moveresize(struct ASMoveResizeData *data, Bool cancelled)
 		asw->frame_canvas->height = 1;
 		
 		on_window_moveresize( asw, asw->frame );
-/*        SendConfigureNotify(asw); */
         broadcast_config (M_CONFIGURE_WINDOW, asw);
     }
     Scr.moveresize_in_progress = NULL ;
@@ -395,6 +366,7 @@ void complete_aswindow_move(struct ASMoveResizeData *data, Bool cancelled)
             LOCAL_DEBUG_OUT( "%dx%d%+d%+d", data->start.width, data->start.height, data->curr.x, data->curr.y);
             moveresize_aswindow_wm( asw, data->curr.x, data->curr.y, data->start.width, data->start.height, False );
         }
+		
         ASWIN_CLEAR_FLAGS( asw, AS_MoveresizeInProgress );
 		asw->frame_canvas->root_x = -10000 ; 
 		asw->frame_canvas->root_y = -10000;
@@ -434,9 +406,52 @@ apply_placement_result( ASStatusHints *status, XRectangle *anchor, ASHints *hint
     if( get_flags( flags, HeightValue ) && height > 0 )
         status->height = height ;
 
-    status2anchor( anchor, hints, status, Scr.VxMax, Scr.VyMax);
+    status2anchor( anchor, hints, status, Scr.VxMax+Scr.MyDisplayWidth, Scr.VyMax+Scr.MyDisplayHeight);
 }
 
+static int
+move_placement_left( ASVector *free_space_list, int x, int y, int w, int h ) 
+{
+    XRectangle *rects = PVECTOR_HEAD(XRectangle,free_space_list);
+    int i = PVECTOR_USED(free_space_list);
+    while( --i >= 0 )
+        if( y >= rects[i].y && y+h <= rects[i].y+(int)rects[i].height && rects[i].x < x && rects[i].x+(int)rects[i].width >= x )
+			x = rects[i].x ;
+	return x ;
+}
+
+static int
+move_placement_right( ASVector *free_space_list, int x, int y, int w, int h ) 
+{
+    XRectangle *rects = PVECTOR_HEAD(XRectangle,free_space_list);
+    int i = PVECTOR_USED(free_space_list);
+    while( --i >= 0 )
+        if( y >= rects[i].y && y+h <= rects[i].y+(int)rects[i].height && rects[i].x <= x+w && rects[i].x+(int)rects[i].width > x+w )
+			x = rects[i].x + (int)rects[i].width  - w ;
+	return x ;
+}
+
+static int
+move_placement_up( ASVector *free_space_list, int x, int y, int w, int h ) 
+{
+    XRectangle *rects = PVECTOR_HEAD(XRectangle,free_space_list);
+    int i = PVECTOR_USED(free_space_list);
+    while( --i >= 0 )
+        if( x >= rects[i].x && x+w <= rects[i].x+(int)rects[i].width  && rects[i].y < y && rects[i].y+(int)rects[i].height >= y )
+			y = rects[i].y ;
+	return y ;
+}
+
+static int
+move_placement_down( ASVector *free_space_list, int x, int y, int w, int h ) 
+{
+    XRectangle *rects = PVECTOR_HEAD(XRectangle,free_space_list);
+    int i = PVECTOR_USED(free_space_list);
+    while( --i >= 0 )
+        if( x >= rects[i].x && x+w <= rects[i].x+(int)rects[i].width  && rects[i].y <= y+h && rects[i].y+(int)rects[i].height > y+h )
+			y = rects[i].y + (int)rects[i].height  - h ;
+	return y ;
+}
 
 static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area )
 {
@@ -446,6 +461,8 @@ static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *
     unsigned short w = asw->status->width;
     unsigned short h = asw->status->height;
     unsigned short dw = w>100?w*5/100:5, dh = h>=100?h*5/100:5 ;
+    int spacer_x = -1 ;
+    int spacer_y = -1 ;
 
     LOCAL_DEBUG_OUT( "size=%dx%d, delta=%dx%d", w, h, dw, dh );
     /* now we have to find the optimal rectangle from the list */
@@ -461,7 +478,49 @@ static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *
             selected = i;
         }
     LOCAL_DEBUG_OUT( "pass1: %d", selected );
-    /* if width < height then swap passes 2 and 3 */
+    
+	if( selected < 0 )
+	{	
+    	i = PVECTOR_USED(free_space_list);	 
+		if( w > 200 && h > 200 )
+		{ /* simply find the biggest rectangle that fits  */
+        	while( --i >= 0 )	
+	        	if( rects[i].width >= w && rects[i].height >= h  )
+            	{
+    	        	if( selected >= 0 )
+			 			if( rects[i].width*rects[i].height <= rects[selected].width*rects[selected].height )			
+							continue;
+					selected = i ;
+				}	 
+			LOCAL_DEBUG_OUT( "pass2a: %d", selected );
+		}else if( w > h ) 
+		{	/* try and fit it by the horizontal edge of the screen */
+        	while( --i >= 0 )	  
+	        	if( rects[i].width >= w && rects[i].height >= h && 
+				    (rects[i].y < 100 || rects[i].y + rects[i].height > area->y+area->height - 100)  )
+				{
+					 selected = i ;	
+					 if( rects[i].y >= 100 ) 
+					 	spacer_y = rects[i].height - h ;
+					 break;
+				}
+			LOCAL_DEBUG_OUT( "pass2b: %d", selected );
+		}else		 
+		{	
+			/* try and fit it by the vertical edge of the screen */
+        	while( --i >= 0 )	  
+	        	if( rects[i].width >= w && rects[i].height >= h && 
+				    (rects[i].x < 100 || rects[i].x + rects[i].width> area->x+area->width - 100)  )
+				{
+					 selected = i ;	
+					 if( rects[i].x >= 100 ) 
+					 	spacer_x = rects[i].width - w ;
+					 break;
+				}
+			LOCAL_DEBUG_OUT( "pass2c: %d", selected );
+		}	
+	}
+	/* if width < height then swap passes 2 and 3 */
     /* pass 2: find rectangle that fits width within margin +- 5% of the window size  and has maximum height
      * left after placement */
     if( selected < 0 )
@@ -533,7 +592,8 @@ static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *
             {
                 int dw = (rects[i].width > w)?rects[i].width - w:1;
                 int dw_sel = (rects[selected].width > w)?rects[selected].width - w:1;
-                if( (rects[i].height - h)/dw > (rects[selected].height - h)/dw_sel  )
+                if( ((rects[i].height - h)*Scr.MyDisplayWidth)/dw > 
+					((rects[selected].height - h)*Scr.MyDisplayWidth)/dw_sel  )
                     selected = i;
             }
     }
@@ -553,7 +613,8 @@ static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *
             {
                 int dh = (rects[i].height > h)?rects[i].height - h:1;
                 int dh_sel = (rects[selected].height > h)?rects[selected].height - h:1;
-                if( (rects[i].width - w)/dh > (rects[selected].width - w)/dh_sel  )
+                if( ((rects[i].width - w)*Scr.MyDisplayHeight)/dh > 
+					((rects[selected].width - w)*Scr.MyDisplayHeight)/dh_sel  )
                     selected = i;
             }
     }
@@ -561,9 +622,82 @@ static Bool do_smart_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *
 
     if( selected >= 0 )
     {
-        int spacer_x = (rects[selected].width > w)? 1: 0;
-        int spacer_y = (rects[selected].height > h)? 1: 0;
-        apply_placement_result_asw( asw, XValue|YValue, rects[selected].x+spacer_x, rects[selected].y+spacer_y, 0, 0 );
+		int target_x, target_y ;
+		int dx, dy ;
+		int move_left, move_up; 
+		Bool changed ; 
+		if( spacer_x < 0 )
+		{	
+			spacer_x = 0 ;
+			if( rects[selected].width > w )
+			{
+				int to_right = (area->x+(int)area->width) - (rects[selected].x + (int)rects[selected].width) ; 
+				if( to_right < rects[selected].x - area->x ) 
+					spacer_x = (int)rects[selected].width - (int)w ; 
+			}	 
+		}
+		if( spacer_y < 0 ) 
+		{	
+			spacer_y = 0 ;
+			if( rects[selected].height > h)
+			{	
+				int to_bottom = (area->y+(int)area->height) - (rects[selected].y + (int)rects[selected].height) ; 
+				if( to_bottom < rects[selected].y - area->y ) 
+					spacer_y = (int)rects[selected].height - (int)h ; 
+			}
+		}
+		target_x = rects[selected].x+spacer_x ;
+		target_y = rects[selected].y+spacer_y ;
+		
+		do 
+		{	
+			int new_x = target_x, new_y = target_y ;
+			changed = False ; 
+			dx = target_x - area->x ; 
+			dy = target_y - area->y ; 
+		
+			move_left = ( dx < area->width - (dx+w) ) ;
+			if( !move_left )
+				dx = area->width - (dx+w);
+
+			move_up = ( dy < area->height - (dy+h) ) ;
+			if( !move_up )
+				dy = area->height - (dy+h);
+			/* we want to place window as close as possible to the edge of the area */
+			if( dx > dy ) 
+			{
+				if( move_left ) 
+					new_x = move_placement_left( free_space_list, target_x, target_y, w, h );
+				else
+					new_x = move_placement_right( free_space_list, target_x, target_y, w, h );
+				if( dy > 0 ) 
+				{	
+					if( move_up ) 
+						new_y = move_placement_up( free_space_list, target_x, target_y, w, h );
+					else
+						new_y = move_placement_down( free_space_list, target_x, target_y, w, h );		
+				}
+			
+			}else if( dy > 0 ) 
+			{
+				if( move_up ) 
+					new_y = move_placement_up( free_space_list, target_x, target_y, w, h );
+				else
+					new_y = move_placement_down( free_space_list, target_x, target_y, w, h );		
+				if( dx > 0 ) 
+				{	
+					if( move_left ) 
+						new_x = move_placement_left( free_space_list, target_x, target_y, w, h );
+					else
+						new_x = move_placement_right( free_space_list, target_x, target_y, w, h );
+				}
+			}
+			LOCAL_DEBUG_OUT( "move_left = %d, move_up = %d, new = %+d%+d, org = %+d%+d", move_left, move_up, new_x, new_y, target_x, target_y );
+			changed = (target_x != new_x || target_y != new_y );
+			target_x = new_x ;
+			target_y = new_y ;
+		}while( changed );
+        apply_placement_result_asw( asw, XValue|YValue, target_x, target_y, 0, 0 );
         LOCAL_DEBUG_OUT( "success: status(%+d%+d), anchor(%+d,%+d)", asw->status->x, asw->status->y, asw->anchor.x, asw->anchor.y );
     }else
     {
@@ -674,6 +808,10 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
     int selected = -1 ;
     unsigned int w = asw->status->width;
     unsigned int h = asw->status->height;
+#ifdef HAVE_XINERAMA
+    unsigned int x = asw->status->x;
+    unsigned int y = asw->status->y;
+#endif    
     ASVector *free_space_list = NULL;
     XRectangle *rects = NULL;
     int i ;
@@ -688,19 +826,25 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
     while( --i >= 0 )
         if( rects[i].width >= w && rects[i].height >=  h )
         {
-            if( selected > 0 )
+          /* if a rect has been selected */  
+	  if( selected > 0 )
             {
-                if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
+              /* if this rect is smaller than the selected one */  
+	      if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
                     continue;
             }
-            selected = i ;
+          /* select this rectangle because it's bigger */  
+	  selected = i ;
         }
-	if( selected  < 0 ) 
+        
+
+        /* if a rect has NOT been selected*/
+        if( selected  < 0 ) 
 	{	                       /* we simply select the biggest area available : */
 	    i = PVECTOR_USED(free_space_list);
 	    while( --i >= 0 )
-       	{
-           	if( selected > 0 )
+	    {
+		if( selected > 0 )
            	{
                	if( rects[i].width * rects[i].height < rects[selected].width * rects[selected].height )
                    	continue;
@@ -708,13 +852,89 @@ do_maximized_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *area)
             selected = i ;
         }
 	}
+
 		 
     if( selected >= 0 )
     {
-        ASFlagType flags = 0 ;
-		int max_width = rects[selected].width ;
-		int max_height = rects[selected].height ;
-        save_aswindow_anchor( asw, ASWIN_GET_FLAGS( asw, AS_MaximizedX), ASWIN_GET_FLAGS( asw, AS_MaximizedY) );
+	    ASFlagType flags = 0 ;
+	    int max_width = rects[selected].width ;
+	    int max_height = rects[selected].height ;
+
+#ifdef HAVE_XINERAMA
+	    /* the following block makes sure windows
+	      are not maximized over multiply heads.*/
+	
+	    int i;
+	    XRectangle *s = Scr.xinerama_screens;
+	    int dest_size = -1;
+	    int dest_rect =  0;
+	    int inter_width;
+	    int inter_height;
+	    
+	    if( s != NULL)
+	    {
+
+	    /* select the xinerama-screen holding most of the
+	      window as it is before maximizing it */
+	    for(i = 0; i< Scr.xinerama_screens_num; ++i)
+	    {
+		    /* if window is not on this xin-rect at all. */
+		    if( ( x < s[i].x && x > s[i].x + s[i].width )||
+			( y < s[i].y && y > s[i].y + s[i].height) )
+			    continue;
+	    
+		    /* window starts left of screen */
+		    if(s[i].x  > x)
+			    /* and ends on the screen */
+			    if( s[i].x + s[i].width > x + w)
+				    inter_width = x + w - s[i].x;
+			    else
+				    inter_width = s[i].width;
+		    else
+			    if( s[i].x + s[i].width > x + w )
+				    inter_width = s[i].width ;
+			    else
+				    inter_width = s[i].x + s[i].width - x;
+		    
+		    
+		    /* window starts above of screen */
+		    if(s[i].y  > y)
+			    /* and ends on the screen */
+			    if( s[i].y + s[i].height > y + h)
+				    inter_height = y + h - s[i].y;
+			    else
+				    inter_height = s[i].height;
+		    else
+			    if( s[i].y + s[i].height > y + h )
+				    inter_height = s[i].height ;
+			    else
+				    inter_height = s[i].y + s[i].height - y;
+		    
+		    if(inter_width * inter_height > dest_size )
+		    {
+			    /* I like this rect better than the last. */
+			    dest_rect = i;
+			    dest_size = inter_width * inter_height;
+		    }
+			    
+		    
+	    }
+    
+	    
+	    if(rects[selected].x < s[dest_rect].x )
+		    rects[selected].x = s[dest_rect].x;
+	    if(rects[selected].y < s[dest_rect].y )
+		    rects[selected].y = s[dest_rect].y;
+	    if( max_width > s[dest_rect].width)
+		    max_width = s[dest_rect].width;
+	    if( max_height > s[dest_rect].height)
+		    max_height = s[dest_rect].height;
+	    }
+
+#endif /* XINERAMA */
+
+	    
+	    save_aswindow_anchor( asw, ASWIN_GET_FLAGS( asw, AS_MaximizedX), ASWIN_GET_FLAGS( asw, AS_MaximizedY) );
 
         if( ASWIN_GET_FLAGS( asw, AS_MaximizedX)  )
             set_flags( flags, XValue|WidthValue );
@@ -759,7 +979,7 @@ static Bool do_tile_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *a
             {
                 if( get_flags( aswbox->flags, ASA_VerticalPriority ) )
                 {
-                    if( get_flags( aswbox->flags, ASA_ReverseOrder ) )
+                    if( get_flags( aswbox->flags, ASA_ReverseOrder ) || get_flags (aswbox->flags, ASA_ReverseOrderH ) )
                     {
                         if( rects[selected].x > rects[i].x )
                             continue;
@@ -772,7 +992,7 @@ static Bool do_tile_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *a
                         if( rects[selected].x == rects[i].x && rects[selected].y < rects[i].y )
                             continue;
                     }
-                }else if( get_flags( aswbox->flags, ASA_ReverseOrder ) )
+                }else if( get_flags( aswbox->flags, ASA_ReverseOrder ) || get_flags( aswbox->flags, ASA_ReverseOrderV ) )
                 {
                     if( rects[selected].y > rects[i].y )
                         continue;
@@ -792,8 +1012,12 @@ static Bool do_tile_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry *a
 
     if( selected >= 0 )
     {
-        int spacer_x = (rects[selected].width > w)? 1: 0;
-        int spacer_y = (rects[selected].height > h)? 1: 0;
+        int spacer_x = aswbox->x_spacing ;
+        int spacer_y = aswbox->y_spacing ;
+		if(rects[selected].width < w+spacer_x)
+			spacer_x = rects[selected].width - w ;
+		if(rects[selected].height < h+spacer_y)
+			spacer_y = rects[selected].height - h ;
         apply_placement_result_asw( asw, XValue|YValue, rects[selected].x+spacer_x, rects[selected].y+spacer_y, 0, 0 );
         LOCAL_DEBUG_OUT( "success: status(%+d%+d), anchor(%+d,%+d)", asw->status->x, asw->status->y, asw->anchor.x, asw->anchor.y );
     }else
@@ -810,10 +1034,20 @@ static Bool do_cascade_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
     int newpos = aswbox->cascade_pos + 25;
     int x = newpos, y = newpos;
 
-    if( get_flags( aswbox->flags, ASA_ReverseOrder ) )
+    if( get_flags( aswbox->flags, ASA_ReverseOrder ) ||
+        ( get_flags( aswbox->flags, ASA_ReverseOrderV ) &&
+          get_flags( aswbox->flags, ASA_ReverseOrderH ) ) ) 
     {
         x = ((int)(area->width) + area->x) - newpos ;
         y = ((int)(area->height) + area->y) - newpos ;
+    }else if( get_flags( aswbox->flags, ASA_ReverseOrderV ) )
+    {
+        x = newpos + area->x ;
+        y = ((int)(area->height) + area->y) - newpos ;
+    }else if( get_flags( aswbox->flags, ASA_ReverseOrderH ) )
+    {
+        x = ((int)(area->width) + area->x) - newpos ;
+        y = newpos + area->y ;
     }else
     {
         x = newpos + area->x ;
@@ -854,6 +1088,7 @@ static Bool do_manual_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
  */
     if( asw->status->width*asw->status->height < (Scr.Feel.OpaqueMove*Scr.MyDisplayWidth*Scr.MyDisplayHeight) / 100 )
     {
+		LOCAL_DEBUG_OUT( "Mapping client window %lX", asw->client_canvas->w );
         map_canvas_window( asw->client_canvas, True );
         map_canvas_window( asw->frame_canvas, False );
 		if( get_desktop_cover_window() != None ) 
@@ -875,11 +1110,7 @@ static Bool do_manual_placement( ASWindow *asw, ASWindowBox *aswbox, ASGeometry
                                         complete_aswindow_move );
     if( mvrdata )
     {
-		raise_scren_panframes( ASDefaultScr );
-        mvrdata->below_sibling = get_lowest_panframe(ASDefaultScr);
-        set_moveresize_restrains( mvrdata, asw->hints, asw->status);
-        mvrdata->grid = make_desktop_grid( ASWIN_DESK(asw), ASWIN_LAYER(asw), False, Scr.Vx, Scr.Vy, asw );
-        Scr.moveresize_in_progress = mvrdata ;
+		setup_aswindow_moveresize(asw, mvrdata );
         InteractiveMoveLoop ();
     }else
         ASWIN_CLEAR_FLAGS( asw, AS_MoveresizeInProgress );
@@ -975,11 +1206,11 @@ place_aswindow_in_windowbox( ASWindow *asw, ASWindowBox *aswbox, ASUsePlacementS
                 return False;
         }
         if( area.width <= 0 )
-            area.width = Scr.MyDisplayWidth ;
+            area.width = (Scr.VxMax + Scr.MyDisplayWidth)- area.x ;
         else if( area.x + area.width > Scr.VxMax + Scr.MyDisplayWidth )
             area.width = Scr.VxMax + Scr.MyDisplayWidth - area.x ;
         if( area.height <= 0 )
-            area.height = Scr.MyDisplayHeight ;
+            area.height = (Scr.VyMax + Scr.MyDisplayHeight)- area.y ;
         else if( area.y + area.height > Scr.VyMax + Scr.MyDisplayHeight )
             area.height = Scr.VyMax + Scr.MyDisplayHeight - area.y ;
     }
@@ -1043,9 +1274,6 @@ Bool place_aswindow( ASWindow *asw )
                      asw->status->width, asw->status->height, asw->status->x, asw->status->y,
                      asw->anchor.width, asw->anchor.height, asw->anchor.x, asw->anchor.y );
 
-    if( ASWIN_GET_FLAGS(asw, AS_MaximizedX|AS_MaximizedY ) )
-        return place_aswindow_in_windowbox( asw, Scr.Feel.default_window_box, ASP_UseBackupStrategy, True );
-
     if( asw->hints->windowbox_name )
     {
         aswbox = find_window_box( &(Scr.Feel), asw->hints->windowbox_name );
@@ -1057,27 +1285,72 @@ Bool place_aswindow( ASWindow *asw )
         }
     }
 
+    /* search for a window-box if none was specified.*/
     if( aswbox == NULL )
     {
-        int i = Scr.Feel.window_boxes_num;
-        aswbox = &(Scr.Feel.window_boxes[0]);
-        while( --i >= 0 )
+        int i, t;
+	ASWindowBox **aswbox_sorted = safemalloc(sizeof(ASWindowBox *) * Scr.Feel.window_boxes_num);
+	aswbox = &(Scr.Feel.window_boxes[0]);
+	
+	/* the following code will make sure window-boxes which have the
+	   virtual-flag set and are not on the current viewport will
+	   be considered for placement after all others. */
+	t = Scr.Feel.window_boxes_num;
+	for( i = 0; i < Scr.Feel.window_boxes_num; ++i)
+	{
+		if( get_flags(aswbox[i].flags, ASA_Virtual)
+		    && (( asw->status->viewport_x / Scr.MyDisplayWidth
+			  != aswbox[i].area.x / Scr.MyDisplayWidth ) ||
+			( asw->status->viewport_y / Scr.MyDisplayHeight
+			  != aswbox[i].area.y / Scr.MyDisplayHeight) )
+		  )
+			/* place window-box at the end */
+			aswbox_sorted[--t] = &aswbox[i];
+		else
+			/* place window-box at the front */
+			aswbox_sorted[i - Scr.Feel.window_boxes_num + t ] = &aswbox[i];
+	}
+	
+        for( i = 0 ; i < Scr.Feel.window_boxes_num ; ++i )
         {
 			LOCAL_DEBUG_OUT("window_box \"%s\": main_strategy = %d, backup_strategy = %d", 
-		   					aswbox[i].name, aswbox[i].main_strategy, aswbox[i].backup_strategy);
+		   					aswbox_sorted[i]->name, aswbox_sorted[i]->main_strategy, aswbox_sorted[i]->backup_strategy);
 			
-            if( IsValidDesk(aswbox[i].desk) && aswbox[i].desk != asw->status->desktop )
+            if( IsValidDesk(aswbox_sorted[i]->desk) && aswbox_sorted[i]->desk != asw->status->desktop )
                 continue;
-            if( aswbox[i].min_layer > asw->status->layer || aswbox[i].max_layer < asw->status->layer )
+            if( aswbox_sorted[i]->min_layer > asw->status->layer || aswbox_sorted[i]->max_layer < asw->status->layer )
                 continue;
-            if( aswbox[i].min_width > asw->status->width || (aswbox[i].max_width > 0 && aswbox[i].max_width < asw->status->width) )
-                continue;
-            if( aswbox[i].min_height > asw->status->height || (aswbox[i].max_height > 0 && aswbox[i].max_height < asw->status->height) )
+            if( aswbox_sorted[i]->min_width > asw->status->width || (aswbox_sorted[i]->max_width > 0 && aswbox_sorted[i]->max_width < asw->status->width) )
                 continue;
+            if( aswbox_sorted[i]->min_height > asw->status->height || (aswbox_sorted[i]->max_height > 0 && aswbox_sorted[i]->max_height < asw->status->height) )
+	        continue;
 
-            if( place_aswindow_in_windowbox( asw, &(aswbox[i]), ASP_UseMainStrategy , False ))
-                return True;
-        }
+
+		    if( ASWIN_GET_FLAGS(asw, AS_MaximizedX|AS_MaximizedY ) )
+			{
+			  int win_x = get_flags(aswbox_sorted[i]->flags, ASA_Virtual) ? asw->status->viewport_x: asw->status->x;
+			  int win_y = get_flags(aswbox_sorted[i]->flags, ASA_Virtual) ? asw->status->viewport_y: asw->status->y;
+			  
+			  
+			  if( aswbox_sorted[i]->area.x > win_x +(int)(asw->status->width) || 
+					aswbox_sorted[i]->area.y > win_y +(int)(asw->status->height)||
+					aswbox_sorted[i]->area.x+(int)aswbox[i].area.width < win_x ||
+					aswbox_sorted[i]->area.y+(int)aswbox[i].area.height < win_y )
+					continue;
+			}	 
+
+		    if( ASWIN_GET_FLAGS(asw, AS_MaximizedX|AS_MaximizedY ) )
+		    {
+			    Bool retval = place_aswindow_in_windowbox( asw, aswbox_sorted[i], ASP_UseBackupStrategy, True );
+			    free( aswbox_sorted );
+			    return retval;
+		    }			
+		    else if( place_aswindow_in_windowbox( asw, aswbox_sorted[i], ASP_UseMainStrategy , False )){
+			    free( aswbox_sorted );
+			    return True;
+		    }        
+	}
+	free( aswbox_sorted );
     }
     return place_aswindow_in_windowbox( asw, Scr.Feel.default_window_box, ASP_UseBackupStrategy, True );
 }
@@ -1145,7 +1418,7 @@ avoid_covering_aswin_iter_func(void *data, void *aux_data)
     return True;
 }
 
-void enforce_avoid_cover(ASWindow *asw )
+void do_enforce_avoid_cover(ASWindow *asw )
 {
 	if( asw && ASWIN_HFLAGS( asw, AS_AvoidCover|AS_ShortLived ) == AS_AvoidCover )
 	{
@@ -1181,6 +1454,29 @@ void enforce_avoid_cover(ASWindow *asw )
 	}
 }
 
+void
+delayed_enforce_avoid_cover( void* vdata )
+{
+	ASWindow *asw = (ASWindow*)vdata;
+	if( asw && asw->magic == MAGIC_ASWINDOW ) 
+		do_enforce_avoid_cover( asw );
+}
+
+void
+enforce_avoid_cover( ASWindow *asw )
+{
+	/* we do not want to enforce avoid cover right away - clients may want to reposition 
+	   themselves automatically */
+
+	if( asw ) 
+		if( ASWIN_GET_FLAGS(asw, AS_AvoidCover|AS_ShortLived) == AS_AvoidCover )
+		{
+			while( timer_remove_by_data( (void*)asw ) );
+			timer_new (500, delayed_enforce_avoid_cover, (void*)asw);	
+		}
+}
+
+
 void obey_avoid_cover(ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_anchor )
 {
     if( asw )
@@ -1188,6 +1484,8 @@ void obey_avoid_cover(ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_
 	    ASWindowBox aswbox ;
         int selected_x = 0 ;
         int selected_y = 0 ;
+		int left = Scr.Vx, right = Scr.Vx+Scr.MyDisplayWidth ;
+		int top = Scr.Vy, bottom = Scr.Vy+Scr.MyDisplayHeight ;
 
         /* we need to move all the res of the window out of the area occupied by us */
 		LOCAL_DEBUG_OUT( "status = %dx%d%+d%+d, layer = %d", asw->status->width, asw->status->height, asw->status->x, asw->status->y, ASWIN_LAYER(asw) );
@@ -1197,10 +1495,28 @@ void obey_avoid_cover(ASWindow *asw, ASStatusHints *tmp_status, XRectangle *tmp_
      *   if all fails we apply backup strategy of the default windowbox
      */
         aswbox.name = mystrdup("default");
+		if( !ASWIN_GET_FLAGS(asw, AS_Sticky))
+		{
+			if( asw->status->x < 0 )
+				left = 0 ; 
+			if( asw->status->x+(int)asw->status->width >= Scr.MyDisplayWidth )		   
+				right = Scr.VxMax + Scr.MyDisplayWidth ; 
+			if( asw->status->y < 0 )
+				top = 0 ; 
+			if( asw->status->y+(int)asw->status->height >= Scr.MyDisplayHeight )		   
+				bottom = Scr.VyMax + Scr.MyDisplayHeight ; 
+		}	 
+#if 0
 		aswbox.area.x = Scr.Vx;
 		aswbox.area.y = Scr.Vy ;
         aswbox.area.width = Scr.MyDisplayWidth ;
         aswbox.area.height = Scr.MyDisplayHeight ;
+#else
+		aswbox.area.x = left;
+		aswbox.area.y = top;
+        aswbox.area.width = right - left ;
+        aswbox.area.height = bottom - top ;
+#endif
         aswbox.main_strategy = ASP_Manual ;
         aswbox.backup_strategy = ASP_Manual ;
         /* we should enforce this one : */
@@ -1332,6 +1648,7 @@ PlaceWindow (ASWindow * tmp_win, unsigned long tflag, int Desk)
 
 		for (i = 0; i < Scr.xinerama_screens_num; ++i)
 		{
+			/* if window is completely in this xinerama-screen */
 			if (s[i].x < x + width && s[i].x + s[i].width > x &&
 				s[i].y < y + height && s[i].y + s[i].height > y)
 			{
